#! /usr/bin/perl -w
#
# Just for fun...
# Originally it was torrentsniff <http://www.highprogrammer.com/alan/perl/torrentsniff.html>
#
# Very simple for use: you need enter only how many attempts send upload data to tracker
# 30 minutes - 1 attempt
# minium size = for less than 4gb: torrent size / 100 || for other: torrent size / 600
# maximum size = minimum * 2

use Digest::SHA1 qw(sha1);
use BDecode;

use LWP::UserAgent;
use HTTP::Request;

use strict;

# Run :-D
main();

exit 0;

# Main
sub main {
	local $| = 1;

	if ( not @ARGV ) {
		usage_exit();
	}

	process_file(@ARGV);
	print "\n";
}

# Open torrent file
sub process_file {
	my($file) = @_;
	print "Torrent file:   $file\n";
	local *TOR;
	if ( not open(TOR, "<", $file) ) {
		print STDERR "Unable to read $file because: $!\n";
		return 0;
	}
	binmode(TOR);
	my $body;
	read(TOR, $body, (-s $file));
	close(TOR);
	output_torrent_data($body);
	sending_torrent_data($body);
}

# Print data of torrent file
sub output_torrent_data {
	my($body) = @_;
	my $result = process_torrent_data($body);
	if ( not defined $result ) {
		print STDERR "Problem reading torrent file\n";
	}

	print  "Info hash:      $result->{'hash'}\n";
	print  "Announce url:   $result->{'announce'}\n";

	my $total = commify($result->{'total_size'});
	print "Size:           $total\n";
}

# Send torrent data to tracker
sub sending_torrent_data {
    my($data) = @_;
    my $result = process_torrent_data($data);

    # Minimum size for upload...
    my $x = int(600);
    if ( $result->{'total_size'} < 4294967296 ) { $x = int(100); }
    my $xy = int($result->{'total_size'}/int($x));

    print "Min size:       ", int($xy/1048576), " mb\n";
    print "Max size:       ", int($xy*2/1048576), " mb\n";
    print "Attempts:       ";
    my $attempts = <STDIN>;
    if (( not $attempts =~ /^\d+$/ ) || ( $attempts == 0 )) {print "Must be numeric and >0\n"; exit 0;}

    my $tor_announce = $result->{'announce'};
    my $tor_info_hash = $result->{'fhash'};
    #transmission 2420
    my $tor_peer_id = "-TR2420-e3f2cdtpq6z0";
    my $tor_port = "56000";
    my $tor_uploaded = "0";
    my $tor_uploaded_start = "0";
    my $tor_uploaded_stop = "0";
    my $tor_downloaded = "0";
    my $tor_left = "0";
    my $tor_event_start = "started";
    my $tor_event_stop = "stopped";
    my $url_start = "$tor_announce&info_hash=$tor_info_hash&peer_id=$tor_peer_id&port=$tor_port&uploaded=$tor_uploaded_start&downloaded=$tor_downloaded&lef=$tor_left&event=$tor_event_start";
    my $url_stop = "$tor_announce&info_hash=$tor_info_hash&peer_id=$tor_peer_id&port=$tor_port&uploaded=$tor_uploaded_stop&downloaded=$tor_downloaded&lef=$tor_left&event=$tor_event_stop";

    my $lwp = LWP::UserAgent->new;
    print "Started...time needed: ", $attempts*30, " minutes\n";

    for(1..$attempts) {
	$tor_uploaded_stop = int(rand($xy*2)) + $xy;
	my $r = HTTP::Request->new(GET => "$url_start");
	my $response = $lwp->request($r);
	if ( not $response->is_success ) {
	    print $response->error_as_HTML;
	}
	print "Waiting 30 minutes...\n";
	sleep(1800);
	print "Sending: ", int($tor_uploaded_stop/1048576). " mb || Uploaded: ", int($tor_uploaded/1048576), " mb\n";
	$tor_uploaded = $tor_uploaded + $tor_uploaded_stop;
	$url_stop = "$tor_announce&info_hash=$tor_info_hash&peer_id=$tor_peer_id&port=$tor_port&uploaded=$tor_uploaded_stop&downloaded=$tor_downloaded&lef=$tor_left&event=$tor_event_stop";
	$r = HTTP::Request->new(GET => "$url_stop");
	$response = $lwp->request($r);
	if ( not $response->is_success ) {
	    print $response->error_as_HTML;
	}
    }
    print "Stopped...uploaded: ", int($tor_uploaded/1048576)," mb\n";
}

# Process torrent file
sub process_torrent_data {
	my($body) = @_;

	my %result;

	my $t = BitTorrent::BDecode::bdecode(\$body);

	my $info = $t->{'info'};
	my $s = substr($body, $t->{'_info_start'}, $t->{'_info_length'});
	my $fhash = sha1($s);
	my $hash = bin2hex(sha1($s));
	my $announce = $t->{'announce'};

	$result{'hash'} = $hash;
	$result{'fhash'} = $fhash;
	$result{'announce'} = $announce;
	$result{'files'} = [];
	my $tsize = 0;
	if ( defined($info->{'files'}) ) {
	    foreach my $f (@{$info->{'files'}}) {
		my %file_record = ( 'size' => $f->{'length'});
		$tsize += $f->{'length'};
		push @{$result{'files'}}, \%file_record;
	    }
	} else {
	    $tsize += $info->{'length'},
	    push @{$result{'files'}}, {'size' => $info->{'length'},};
	}
	$result{'total_size'} = $tsize;
	return \%result;
}

# Add commas to number
sub commify {
 local $_ = shift;
  1 while s/^([-+]?\d+)(\d{3})/$1,$2/;
  return $_;
}

# Encode binary as hex characters
sub bin2hex {
  my ($d) = @_;
  $d =~ s/(.)/sprintf("%02x",ord($1))/egs;
  $d = lc($d);
  return $d;
}

sub usage_exit {
	print <<END;
$0 [.torrent file]
END
	exit 1;
}
